from invenio_pidstore.providers.recordid_v2 import RecordIdProviderV2
from invenio_records.systemfields import ConstantField, RelationsField
from invenio_records_resources.records.api import Record as InvenioBaseRecord
from invenio_records_resources.records.systemfields import IndexField
from invenio_records_resources.records.systemfields.pid import PIDField, PIDFieldContext
from oarepo_vocabularies.records.api import OARepoVocabularyBase

from nr_vocabularies.records.dumper import NRVocabulariesDumper
from nr_vocabularies.records.models import NRVocabulariesMetadata


class NRVocabulary(OARepoVocabularyBase, InvenioBaseRecord):
    model_cls = NRVocabulariesMetadata
    schema = ConstantField("$schema", "local://nr-vocabularies-1.0.0.json")
    index = IndexField("nr_vocabularies-nr-vocabularies-1.0.0")

    dumper_extensions = [*OARepoVocabularyBase.dumper_extensions]
    dumper = NRVocabulariesDumper(extensions=dumper_extensions)



from invenio_records_resources.resources import RecordResource as InvenioRecordResource
from oarepo_vocabularies.resources.resource import OARepoVocabulariesResourceBase


class NRVocabulariesResource(OARepoVocabulariesResourceBase, InvenioRecordResource):
    """NRVocabulary resource."""

    # here you can for example redefine
    # create_url_rules function to add your own rules



from invenio_records_resources.services import SearchOptions as InvenioSearchOptions

from . import facets


def _(x):
    """Identity function for string extraction."""
    return x


from oarepo_vocabularies.services.search import OARepoVocabulariesSearchOptionsBase


class NRVocabulariesSearchOptions(
    OARepoVocabulariesSearchOptionsBase, InvenioSearchOptions
):
    """NRVocabulary search options."""

    facets = {
        "relatedURI_COAR": facets.relatedURI_COAR,
        "relatedURI_CrossrefFunderID": facets.relatedURI_CrossrefFunderID,
        "relatedURI_ROR": facets.relatedURI_ROR,
        "relatedURI_URL": facets.relatedURI_URL,
        "marcCode": facets.marcCode,
        "dataCiteCode": facets.dataCiteCode,
        "alpha3Code": facets.alpha3Code,
        "alpha3CodeENG": facets.alpha3CodeENG,
        "alpha3CodeNative": facets.alpha3CodeNative,
        "acronym": facets.acronym,
        "RID": facets.RID,
        "ICO": facets.ICO,
        "coarType": facets.coarType,
        "dataCiteType": facets.dataCiteType,
        "nameType": facets.nameType,
        "nonpreferredLabels_cs": facets.nonpreferredLabels_cs,
        "nonpreferredLabels_en": facets.nonpreferredLabels_en,
        "pair": facets.pair,
        "uuid": facets.uuid,
        "indexed_at": facets.indexed_at,
        "created": facets.created,
        "updated": facets.updated,
        "type_pid_type": facets.type_pid_type,
        "type_id": facets.type_id,
        "pid_pk": facets.pid_pk,
        "pid_pid_type": facets.pid_pid_type,
        "pid_obj_type": facets.pid_obj_type,
        "pid_status": facets.pid_status,
        "title_sort": facets.title_sort,
        "icon": facets.icon,
        "hierarchy_level": facets.hierarchy_level,
        "_id": facets._id,
        "_schema": facets._schema,
        "contexts": facets.contexts,
        "tags": facets.tags,
    }
    sort_options = {
        **OARepoVocabulariesSearchOptionsBase.sort_options,
    }



"""Facet definitions."""

from elasticsearch_dsl import Facet
from elasticsearch_dsl.query import Nested
from invenio_records_resources.services.records.facets import TermsFacet


class NestedLabeledFacet(Facet):
    agg_type = "nested"

    def __init__(self, path, nested_facet, label=""):
        self._path = path
        self._inner = nested_facet
        self._label = label
        super(NestedLabeledFacet, self).__init__(
            path=path,
            aggs={
                "inner": nested_facet.get_aggregation(),
            },
        )

    def get_values(self, data, filter_values):
        return self._inner.get_values(data.inner, filter_values)

    def add_filter(self, filter_values):
        inner_q = self._inner.add_filter(filter_values)
        if inner_q:
            return Nested(path=self._path, query=inner_q)

    def get_labelled_values(self, data, filter_values):
        """Get a labelled version of a bucket."""
        try:
            out = data["buckets"]
        except:
            out = []
        return {"buckets": out, "label": str(self._label)}


relatedURI_COAR = TermsFacet(field="relatedURI.COAR")


relatedURI_CrossrefFunderID = TermsFacet(field="relatedURI.CrossrefFunderID")


relatedURI_ROR = TermsFacet(field="relatedURI.ROR")


relatedURI_URL = TermsFacet(field="relatedURI.URL")


marcCode = TermsFacet(field="marcCode")


dataCiteCode = TermsFacet(field="dataCiteCode")


alpha3Code = TermsFacet(field="alpha3Code")


alpha3CodeENG = TermsFacet(field="alpha3CodeENG")


alpha3CodeNative = TermsFacet(field="alpha3CodeNative")


acronym = TermsFacet(field="acronym")


RID = TermsFacet(field="RID")


ICO = TermsFacet(field="ICO")


coarType = TermsFacet(field="coarType")


dataCiteType = TermsFacet(field="dataCiteType")


nameType = TermsFacet(field="nameType")


nonpreferredLabels_cs = TermsFacet(field="nonpreferredLabels.cs")


nonpreferredLabels_en = TermsFacet(field="nonpreferredLabels.en")


pair = TermsFacet(field="pair")


uuid = TermsFacet(field="uuid")


indexed_at = TermsFacet(field="indexed_at")


created = TermsFacet(field="created")


updated = TermsFacet(field="updated")


type_pid_type = TermsFacet(field="type.pid_type")


type_id = TermsFacet(field="type.id")


pid_pk = TermsFacet(field="pid.pk")


pid_pid_type = TermsFacet(field="pid.pid_type")


pid_obj_type = TermsFacet(field="pid.obj_type")


pid_status = TermsFacet(field="pid.status")


title_sort = TermsFacet(field="title_sort")


icon = TermsFacet(field="icon")


hierarchy_level = TermsFacet(field="hierarchy.level")


_id = TermsFacet(field="id")


_schema = TermsFacet(field="$schema")


contexts = TermsFacet(field="contexts")


tags = TermsFacet(field="tags")



from flask import current_app
from werkzeug.local import LocalProxy


def _ext_proxy(attr):
    return LocalProxy(lambda: getattr(current_app.extensions["nr-vocabularies"], attr))


current_service = _ext_proxy("service")
"""Proxy to the instantiated vocabulary service."""


current_resource = _ext_proxy("resource")
"""Proxy to the instantiated vocabulary resource."""



from invenio_records_resources.services import RecordLink
from invenio_records_resources.services import (
    RecordServiceConfig as InvenioRecordServiceConfig,
)
from invenio_records_resources.services import pagination_links
from oarepo_vocabularies.services.config import OARepoVocabulariesServiceConfigBase

from nr_vocabularies.records.api import NRVocabulary
from nr_vocabularies.services.permissions import NRVocabulariesPermissionPolicy
from nr_vocabularies.services.schema import NRVocabularySchema
from nr_vocabularies.services.search import NRVocabulariesSearchOptions


class NRVocabulariesServiceConfig(
    OARepoVocabulariesServiceConfigBase, InvenioRecordServiceConfig
):
    """NRVocabulary service config."""

    permission_policy_cls = NRVocabulariesPermissionPolicy
    schema = NRVocabularySchema
    search = NRVocabulariesSearchOptions
    record_cls = NRVocabulary

    components = [*OARepoVocabulariesServiceConfigBase.components]

    model = "nr_vocabularies"



import invenio_vocabularies.services.schema as vocabularies_schema
import marshmallow as ma
import marshmallow.fields as ma_fields
import marshmallow.validate as ma_valid
from invenio_records_resources.services.records.schema import (
    BaseRecordSchema as InvenioBaseRecordSchema,
)
from invenio_vocabularies.services.schema import BaseVocabularySchema
from marshmallow import ValidationError
from marshmallow import validates as ma_validates


class RelatedURISchema(
    ma.Schema,
):
    """RelatedURISchema schema."""

    COAR = ma_fields.String()

    CrossrefFunderID = ma_fields.String()

    ROR = ma_fields.String()

    URL = ma_fields.String()


class NonPreferredLabels(
    ma.Schema,
):
    """NonPreferredLabels schema."""

    cs = ma_fields.List(ma_fields.String())

    en = ma_fields.List(ma_fields.String())


class NRVocabularySchema(
    BaseVocabularySchema,
):
    """NRVocabularySchema schema."""

    relatedURI = ma_fields.Nested(RelatedURISchema)

    marcCode = ma_fields.String()

    dataCiteCode = ma_fields.String()

    alpha3Code = ma_fields.String()

    alpha3CodeENG = ma_fields.String()

    alpha3CodeNative = ma_fields.String()

    acronym = ma_fields.String()

    RID = ma_fields.String()

    ICO = ma_fields.String()

    coarType = ma_fields.String()

    dataCiteType = ma_fields.String()

    nameType = ma_fields.String()

    nonpreferredLabels = ma_fields.Nested(NonPreferredLabels)

    pair = ma_fields.String()

    hint = vocabularies_schema.i18n_strings

    type = ma_fields.Str(required=True, attribute="type.id")

    contexts = ma_fields.List(ma_fields.String())

    tags = ma_fields.List(ma_fields.String())



from invenio_records_resources.services import RecordService as InvenioRecordService
from oarepo_vocabularies.services.service import OARepoVocabulariesServiceBase


class NRVocabulariesService(OARepoVocabulariesServiceBase, InvenioRecordService):
    """NRVocabulary service."""



def create_blueprint_from_app(app):
    """Create  blueprint."""
    blueprint = app.extensions["nr-vocabularies"].resource.as_blueprint()
    blueprint.record_once(init)
    return blueprint


def init(state):
    """Init app."""
    app = state.app
    ext = app.extensions["nr-vocabularies"]

    # register service
    sregistry = app.extensions["invenio-records-resources"].registry
    sregistry.register(ext.service, service_id="nr-vocabularies")

    # Register indexer
    iregistry = app.extensions["invenio-indexer"].registry
    iregistry.register(ext.service.indexer, indexer_id="nr-vocabularies")



from invenio_db import db
from invenio_records.models import RecordMetadataBase
from oarepo_vocabularies.records.models import OARepoVocabularyMetadataBase


class NRVocabulariesMetadata(
    OARepoVocabularyMetadataBase, db.Model, RecordMetadataBase
):
    """Model for NRVocabulary metadata."""

    __tablename__ = "nrvocabularies_metadata"

    # Enables SQLAlchemy-Continuum versioning
    __versioned__ = {}



from nr_vocabularies.resources.config import NRVocabulariesResourceConfig
from nr_vocabularies.resources.resource import NRVocabulariesResource
from nr_vocabularies.services.config import NRVocabulariesServiceConfig
from nr_vocabularies.services.service import NRVocabulariesService

NR_VOCABULARIES_RESOURCE_CONFIG = NRVocabulariesResourceConfig
NR_VOCABULARIES_RESOURCE_CLASS = NRVocabulariesResource
NR_VOCABULARIES_SERVICE_CONFIG = NRVocabulariesServiceConfig
NR_VOCABULARIES_SERVICE_CLASS = NRVocabulariesService



from invenio_records_resources.resources import (
    RecordResourceConfig as InvenioRecordResourceConfig,
)
from oarepo_vocabularies.resources.config import OARepoVocabulariesResourceConfigBase


class NRVocabulariesResourceConfig(
    OARepoVocabulariesResourceConfigBase, InvenioRecordResourceConfig
):
    """NRVocabulary resource config."""

    blueprint_name = "NRVocabularies"
    url_prefix = "/v/"



from invenio_records.dumpers import ElasticsearchDumper as InvenioElasticsearchDumper
from oarepo_vocabularies.records.dumper import OARepoVocabularyDumperBase


class NRVocabulariesDumper(OARepoVocabularyDumperBase, InvenioElasticsearchDumper):
    """NRVocabulary elasticsearch dumper."""
